/* WARNING! This file was autogenerated from template */
/****************************************************************************/
/*                                                                          */
/*  This file is part of QSopt_ex.                                          */
/*                                                                          */
/*  (c) Copyright 2006 by David Applegate, William Cook, Sanjeeb Dash,      */
/*  and Daniel Espinoza                                                     */
/*                                                                          */
/*  Sanjeeb Dash ownership of copyright in QSopt_ex is derived from his     */
/*  copyright in QSopt.                                                     */
/*                                                                          */
/*  This code may be used under the terms of the GNU General Public License */
/*  (Version 2.1 or later) as published by the Free Software Foundation.    */
/*                                                                          */
/*  Alternatively, use is granted for research purposes only.               */
/*                                                                          */
/*  It is your choice of which of these two licenses you are operating      */
/*  under.                                                                  */
/*                                                                          */
/*  We make no guarantees about the correctness or usefulness of this code. */
/*                                                                          */
/****************************************************************************/

#ifndef mpf_PRESOLVE_H
#define mpf_PRESOLVE_H

#include "lpdata_mpf.h"

void mpf_ILLlp_sinfo_init(mpf_ILLlp_sinfo * sinfo);
void mpf_ILLlp_sinfo_free(mpf_ILLlp_sinfo * sinfo);
int mpf_ILLlp_sinfo_print(mpf_ILLlp_sinfo * s);

void mpf_ILLlp_predata_init(mpf_ILLlp_predata * pre);
void mpf_ILLlp_predata_free(mpf_ILLlp_predata * pre);

void mpf_ILLlp_preop_init(mpf_ILLlp_preop * op);
void mpf_ILLlp_preop_free(mpf_ILLlp_preop * op);

void mpf_ILLlp_preline_init(mpf_ILLlp_preline * line);
void mpf_ILLlp_preline_free(mpf_ILLlp_preline * line);

#endif /* ! mpf_PRESOLVE_H */
