/* QSopt_ex_version.h -- QSopt_ex version information header
 * This file is part of QSopt_ex.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public 
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Copyright (c) 2014  Jon Lund Steffensen <jonlst@gmail.com> 
 */

#ifndef QSOPT_EX_VERSION_H
#define QSOPT_EX_VERSION_H

void QSopt_ex_version(void);

#endif /* !QSOPT_EX_VERSION_H */
